# Amiga Library Services - CD Administration Makefile
#
# This Makefile goes in the root directory of the "New Material" tree, and
# is responsible for doing such things as:
#
# (1)	Build an accumulated "Contents" file for all material in this tree.
#
# (2)	Build a "Quick Reference" file for all material in this tree.
#
# (3)	Build archives for the material in this tree on a "per submission"
#	basis, and install them in the BBS tree on this CD, along with
#	appropriate product info files.
#

SHELL=/bin/sh
CONTENTS_FORMAT = "==========\n\n%N    %V    %S\n\n%d\n\nAuthor: %a\nPath: %P\n\n"
QUICKREF_FORMAT = "%-16.16N  %8.8V   %-S\n"


# This is the default thing to do.

all:	Contents QuickRef AllList


# Generate a summary file called "Contents".  Note that the entries in the
# contents are generated in pathname sorted, case independent, order.

Contents:
	echo >>$@ "This file is generated automatically from the product info files"
	echo >>$@ "included in this tree."
	echo >>$@ ""
	find . \( -name Product-Info -o -name '*.pi' \) -print \
	  | sort -f \
	  | sed "s:^./::" \
	  | pitool -f - -b -F $(CONTENTS_FORMAT) - >>$@
	if test  -f $@.info; then true; else echo "WARNING - missing $@.info"; fi


# Generate a quick reference file, which contains the name of the product,
# the version number, and the short description; one product per line.

QuickRef:
	echo >>$@ "This file is generated automatically from the product info files"
	echo >>$@ "included in this tree.  See the 'Contents' file for additional"
	echo >>$@ "information about each item."
	echo >>$@ ""
	pitool -b -F $(QUICKREF_FORMAT) -f - . | sort -f | uniq >>$@
	if test -f $@.info; then true; else echo "WARNING - missing $@.info"; fi

AllList:
	find . -print | sed "s:^./::" | sort >/tmp/all.tmp
	cp /tmp/all.tmp $@

BBS:	AllList
	$(SHELL) PackNew.sh AllList

clean:
	rm -f *! *~

# Remove all automatically generated files.

clobber: clean
	rm -f Contents QuickRef AllList

